# coding: utf8
"""This directory contains parsers for several websites.

Parsers are analogous to youtube-dl extractors: they define how to parse out infomation of a manga on a webpage.
Parsers *must* subclass and satisfy the tankobon.manga.Parser interface.

Typically, parsers make heavy use of regexes and leverage the BeautifulSoup API through the .soup attribute:

>>> class MyParser(Parser):
...
...     # The domain of the website you are going to parse, *without* a 'www' in front.
...     domain = "my-website.com"
...
...     def chapters(self):
...         for tag in self.soup.find_all("div"):
...             yield {
...                 "id": tag.h1.text,
...                 "title": tag.h2.text,
...                 "url": tag.a["href"]
...             }
...
...     def pages(self, soup):
...         return [tag["href"] for tag in soup.find_all("a", href=True)]

When subclassed, the new parser will automatically be registered.
The parser will then be delegated to based on the domain name (using urlparse's netloc):

>>> from tankobon import parsers
>>> parser = parsers.load({"url": "https://my-website.com/manga/12345"})
"""

import collections
import importlib
import json
import pathlib

from tankobon import manga, utils

INDENT = 4

# Manga metadata is stored here using a filename-safe version of its title.
CACHE_PATH = pathlib.Path.home() / ".tankobon"
CACHE_PATH.mkdir(exist_ok=True)

# This file maps manga URLs to their filenames.
INDEX_PATH = CACHE_PATH / "_index.json"


def _register():
    # import for side effects (register default parser classes)
    for submodule in (pathlib.Path(__file__).parent / "_parsers").glob("*.py"):
        _ = importlib.import_module(f"tankobon._parsers.{submodule.stem}")


_register()


def _create_filename(parser):
    return utils.sanitize_filename(parser.title())


def load(*args, **kwargs) -> manga.Parser:
    """Load a parser and initalize it with the given arguments.
    The parser to be loaded will be based on the domain of data['url'] (the data argument).


    Args:
        *args: Passed to the parser.
        **kwargs: Passed to the parser.

    Raises:
        ValueError, if the data arg is not passed or the parser for the website's domain does not exist.

    Returns:
        The parser.
    """

    try:
        data = kwargs.get("data") or args[0]
    except IndexError:
        raise ValueError("no data arg (can't determine parser to use without url)")

    domain = utils.parse_domain(data["url"])

    parser = manga._parsers.get(domain)
    if parser is not None:
        return parser(*args, **kwargs)

    # domain does not need to match _exactly_.
    # we don't need to mess around with TLDs, just do a substring check
    for parser_domain, parser in manga._parsers.items():
        if parser_domain in domain:
            return parser(*args, **kwargs)

    # no parser matches
    raise ValueError(f"no parser exists for domain {domain}")


class Index(collections.UserDict):
    """An index to keep track of manga metadata."""

    def __init__(self, path: pathlib.Path = INDEX_PATH):
        self._path = path
        try:
            with self._path.open() as f:
                data = json.load(f)
        except FileNotFoundError:
            data = {}

        super().__init__(data)

    def close(self):
        with self._path.open("w") as f:
            json.dump(self.data, f, indent=INDENT)


class Cache:
    """A cache for manga metadata."""

    def __init__(self, path: pathlib.Path = CACHE_PATH):
        self._path = path
        self._index = Index(self._path / "_index.json")

    def __enter__(self):
        return self

    def __exit__(self, t, v, tb):
        self.close()

    def close(self):
        self._index.close()

    @property
    def manga_names(self):
        return list(self._index.values())

    def load_metadata(self, name: str) -> dict:
        """Load metadata for a manga.

        Args:
            name: The manga name/url.

        Returns:
            The manga metadata or an empty dict if the metadata does not exist.
        """

        if utils.is_url(name):
            filename = self._index.get(name)
        else:
            filename = name

        if filename is not None:
            with (self._path / f"{filename}.json").open() as f:
                return json.load(f)

        return {}

    def load(self, url: str) -> manga.Parser:
        """Initalize a parser by url with its previously cached metadata.

        Args:
            url: The manga url.

        Returns:
            The parser.
        """

        metadata = self.load_metadata(url)
        if not metadata:
            metadata = {"url": url}

        return load(data=metadata)

    def dump(self, parser: manga.Parser) -> None:
        """Cache the updated metadata of a parser.

        Args:
            parser: The manga parser.
        """

        name = _create_filename(parser)
        url = parser.data["url"].rstrip("/")
        self._index[url] = name

        with (self._path / f"{name}.json").open("w") as f:
            json.dump(parser.data, f, indent=INDENT)



# coding: utf8

import re

from tankobon import manga


RE_CHAPTER = re.compile(r".*[Cc]hapter ?([\d\.]+) ?:? ?(.*)")


class Parser(manga.Parser):

    domain = "mangakakalot.com"

    def chapters(self):
        for tag in self.soup.find_all("div", class_="row"):
            link = tag.span.a
            if not link:
                continue

            chapter, title = RE_CHAPTER.findall(link.text)[0]
            yield {"id": chapter, "title": title, "url": link["href"]}

    def pages(self, soup):
        return [
            tag["src"]
            for tag in soup.find("div", class_="container-chapter-reader").find_all(
                "img", src=True
            )
        ]

    def title(self):
        return self.soup.find("ul", class_="manga-info-text").li.h1.text



# coding: utf8
"""Utilities for tankobon.
"""

import logging
import pathlib
import re
from typing import Optional, Union
from urllib.parse import urlparse


import bs4  # type: ignore
import filetype  # type: ignore
import requests

Number = Union[int, float]

_log = logging.getLogger("tankobon")

# Downloader config
BSOUP_PARSER = "html5lib"  # if you want, change to lxml for faster parsing
TIMEOUT = 15
COOLDOWN = 2

RE_DOMAIN = re.compile(r"^(?:www\.)?(.*)(:(\d+))?$")


def filesize(content: bytes) -> str:
    """Create a human-readable filesize for content.

    Args:
        content: The bytes to get the size of.
    Returns:
        A string of the filesize ending in B, kB, etc.
    """
    filesize = float(len(content))
    for suffix in ["B", "KiB", "MiB", "GiB"]:
        if filesize < 1024.0 or suffix == "GiB":
            break
        filesize /= 1024.0
    return f"{filesize:.1f} {suffix}"


def _is_valid_char(char):
    return char.isalnum()


def sanitize_filename(name: str) -> str:
    """Sanitise a name so it can be used as a filename.
    Args:
        name: The name to sanitise.
    Returns:
        The sanitised name as a string.
    """

    sanitised = "".join([c.lower() if _is_valid_char(c) else "_" for c in name])

    # remove duplicate underscores
    return re.sub("_{2,}", "_", sanitised).strip("_")


def get_soup(
    *args,
    encoding: Optional[str] = None,
    parser: str = BSOUP_PARSER,
    session: Optional[requests.Session] = None,
    **kwargs,
) -> bs4.BeautifulSoup:
    """Get a url as a BeautifulSoup.

    Args:
        *args: See get_url.
        encoding: The encoding to decode.
            Defaults to the autodetected encoding (by requests).
        parser: The parser to use.
            Must be 'html.parser', 'html5lib' or 'lxml'.
        session: The session to use to download the soup.
            Defaults to None.
        **kwargs: See get_url.
    """

    if session is not None:
        response = session.get(*args, **kwargs)
    else:
        response = requests.get(*args, **kwargs)

    if encoding is not None:
        html_text = response.content.decode(encoding)
    else:
        html_text = response.text

    return bs4.BeautifulSoup(html_text, parser)


def save_response(path: pathlib.Path, res: requests.models.Response) -> pathlib.Path:
    """Save a Requests response at path with the correct file extension.

    Args:
        path: The path where to save the file at.
        res: The response.

    Returns:
        The full path to the file.
    """

    path = path.with_suffix(f".{filetype.guess(res.content).extension}")
    with path.open("wb") as f:
        f.write(res.content)
    _log.debug(
        "saved response from %s at %s (%s)", res.url, path, filesize(res.content)
    )

    return path


class TimedSession(requests.Session):
    def get(self, *args, **kwargs):
        return super().get(*args, timeout=TIMEOUT, **kwargs)


def is_url(url: str) -> bool:
    """Check whether or not a string is a url."""
    result = urlparse(url)
    return all([result.scheme, result.netloc, result.path])


def parse_domain(url: str) -> str:
    """Parse out a url's domain.

    Args:
        url: The url to parse.

    Returns:
        The domain.
    """

    return RE_DOMAIN.findall(urlparse(url).netloc)[0][0]



# coding: utf8

import re

from tankobon import manga


RE_TITLE = re.compile(r"(.*) Manga *\|")
RE_CHAPTER = re.compile(r"Chapter (\d+)\:? ?([\w \(\)]*)")


class Parser(manga.Parser):

    domain = "mangabat.com"

    def chapters(self):
        for tag in self.soup.find_all("a", class_="chapter-name"):
            href = str(tag.get("href"))
            title = str(tag.text)

            match = RE_CHAPTER.findall(title)

            if match:
                chapter, title = match[0]
                yield {"id": chapter, "title": title, "url": href}

    def pages(self, soup):
        pages_div = soup.find("div", class_="container-chapter-reader")
        if not pages_div:
            return

        return [link["src"] for link in pages_div.find_all("img", src=True)]

    def title(self):
        return RE_TITLE.findall(super().title())[0]

    def description(self):
        desc = self.soup.find("div", class_="panel-story-info-description")
        desc.h3.decompose()
        return desc.get_text(strip=True)



# coding: utf8

import abc
import concurrent.futures as cfutures
import logging
import pathlib
import shutil
import typing as tp

import bs4  # type: ignore
import fake_useragent  # type: ignore

try:
    import fpdf  # type: ignore
    import imagesize  # type: ignore
    import natsort  # type: ignore
except ImportError:
    fpdf = None
    imagesize = None
    natsort = None

from tankobon import utils

# type hints
MangaData = tp.Dict[str, tp.Any]

# constants
BS4_PARSER = "html.parser"
VOLUME_NOTSET = "unknown"

_log = logging.getLogger("tankobon")

_parsers = {}
_useragent = fake_useragent.UserAgent()


class Parser(abc.ABC):
    """A manga website parser.

    Args:
        data: The manga metadata as a dict.
            These keys are required:

            'url' (str): The manga website's url.
        timeout: How long to wait for the website server to respond.
            Defaults to utils.TIMEOUT (5).

    Attributes:
        data: The manga metadata.
        domain: The website domain this parser is intended to parse.
            (i.e google.com)
        DEFAULTS: Default values for the manga metadata.
            On init, the 'data' arg is used to update these defaults into .data.
        session: The requests.Session used to retrieve urls.
        soup: The BeautifulSoup of the manga website.

    Raises:
        ValueError, if the url is invalid.
    """

    domain: tp.Optional[str] = None
    DEFAULTS: tp.Dict[str, tp.Any] = {
        "chapters": {},
        "title": None,
        "url": None,
    }

    def __init__(
        self, data: tp.Optional[MangaData] = None, timeout: int = utils.TIMEOUT
    ) -> None:
        if data is None:
            data = {}

        self.data = self.DEFAULTS
        self.data.update(data)

        url = self.data["url"]

        if not utils.is_url(url):
            raise ValueError(f"invalid url: {url}")

        self.session = utils.TimedSession()
        self.session.headers.update({"Referer": url, "User-Agent": _useragent.random})

        self.soup = self.soup_from_url(url)

    # implicitly register class
    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        if cls.domain is None:
            raise TypeError("'domain' attribute needs to be overriden in subclass")

        _parsers[cls.domain] = cls

    def __enter__(self):
        return self

    def __exit__(self, t, v, tb):
        self.close()

    def close(self) -> None:
        self.session.close()
        self.soup.decompose()

    @abc.abstractmethod
    def chapters(self) -> tp.Generator[tp.Dict[str, str], None, None]:
        """Parse chapters from the webpage.

        Yields:
            A dict in the form:
            (all values must be strings)
            {
                "id": ...,  # chapter 'number'
                "title": ...,
                "url": ...,
                "volume": ...,  # volume 'number'
            }
            where volume is optional and may be undefined.
        """
        pass

    @abc.abstractmethod
    def pages(self, soup: bs4.Tag) -> tp.List[str]:
        """Parse pages from the chapter webpage.

        Args:
            soup: The soup of the url from the chapter data yielded by .chapters().

        Returns:
            A list of direct URLs to the page images in order of page number.
        """
        pass

    def title(self) -> str:
        """Parse the manga title from the webpage.

        Returns:
            The manga title.
        """
        return (
            self.soup.title.text
            or self.soup.find("meta", property="og:title")["content"]
        )

    def description(self) -> str:
        """Parse the manga description.

        Returns:
            The description.
        """
        return ""

    def soup_from_url(self, url: str) -> bs4.BeautifulSoup:
        return bs4.BeautifulSoup(self.session.get(url).content, BS4_PARSER)

    def refresh(self) -> None:
        """Add any new chapters to the metadata."""

        self.data["title"] = self.title()
        self.data["description"] = self.description()

        _log.info("[refresh] adding new chapters")

        for chapter_data in self.chapters():
            chapter_id = chapter_data.pop("id")

            _log.debug("[refresh] adding chapter %s", chapter_id)

            if chapter_id in self.data["chapters"]:
                continue

            if "volume" not in chapter_data:
                _log.warning("[refresh] chapter %s has no volume", chapter_id)
                chapter_data["volume"] = VOLUME_NOTSET

            self.data["chapters"][chapter_id] = chapter_data

    def parse(
        self, force: bool = False, refresh: bool = True, volume: tp.Optional[str] = None
    ) -> None:
        """Parse pages from all the chapters.

        Args:
            force: Whether or not to parse chapters previously parsed.
                Defaults to False.
            refresh: Whether or not to refresh metadata for any new chapters before parsing.
                Defaults to True.
            volume: Parse chapters belonging only to a specific volume.
                If None, all chapters are parsed.
                Defaults to None.
        """

        pool = cfutures.ThreadPoolExecutor()
        results = {}

        if refresh:
            self.refresh()

        for chapter_id, chapter_data in self.data["chapters"].items():

            if chapter_data.get("pages") is not None and not force:
                _log.info("[parse] chapter %s already parsed, skipping", chapter_id)
                continue

            if volume is not None and chapter_data["volume"] != volume:
                _log.info(
                    "[parse] chapter %s not in volume %s, skipping", chapter_id, volume
                )
                continue

            # add task to pool
            _log.info("[parse] submitting task for chapter %s", chapter_id)
            future = pool.submit(self.pages, self.soup_from_url(chapter_data["url"]))
            results[future] = chapter_id

        for future in cfutures.as_completed(results):
            chapter_id = results[future]
            pages = future.result()

            _log.info(
                "[parse] adding %s pages to chapter %s", str(len(pages)), chapter_id
            )
            self.data["chapters"][chapter_id]["pages"] = pages

        _log.info("[parse] done")
        pool.shutdown()


class Downloader:
    def __init__(self, data: MangaData) -> None:
        self.data = data
        self.session = utils.TimedSession()

        self.session.headers.update({"Referer": self.data["url"]})

    def __enter__(self):
        return self

    def __exit__(self, t, v, tb):
        self.close()

    def close(self) -> None:
        self.session.close()

    def download(
        self,
        path: tp.Union[str, pathlib.Path],
        chapters: tp.Optional[tp.List[str]] = None,
        force: bool = False,
        cooldown: int = utils.COOLDOWN,
    ) -> None:
        """Download the manga to disk.
        Each chapter will be downloaded to its own folder (1, 2, 3, etc).

        Args:
            path: Where to download the manga.
            chapters: The chapters to download.
                Defaults to all chapters.
            force: Whether or not to download chapters previously downloaded.
                Defaults to False.
            cooldown: How long to wait between downloading each page.
                Defaults to utils.COOLDOWN.

        Returns:
            A dict map of chapter_id to a list of page filenames (i.e 1.png, 2.png, etc.)
        """

        path = pathlib.Path(path)
        chapters = chapters or list(self.data["chapters"])

        with cfutures.ThreadPoolExecutor() as pool:
            results = {}
            for chapter in chapters:
                chapter_path = path / chapter

                if chapter_path.is_dir():
                    _log.info("[download] chapter %s exists, skipping", chapter)
                    continue

                print(chapter)
                for page_number, page in enumerate(
                    self.data["chapters"][chapter]["pages"]
                ):
                    future = pool.submit(self.session.get, page)

                    # three-tuple of chapter number, page number, and page path
                    results[future] = (
                        chapter,
                        page_number,
                        chapter_path / str(page_number),
                    )

            for future in cfutures.as_completed(results):
                chapter, page, page_path = results[future]

                try:
                    resp = future.result()

                except Exception as e:
                    _log.critical(
                        "[download] could not download page %s for chapter %s: %s",
                        page,
                        chapter,
                        e,
                    )

                    shutil.rmtree(str(page_path.parent))
                    raise e

                else:
                    _log.info("[download] saving page %s of chapter %s", page, chapter)

                    page_path.parent.mkdir(exist_ok=True)
                    utils.save_response(page_path, resp)

    def export_pdf(self, volume: str, to: tp.Union[str, pathlib.Path]) -> None:
        """Export the manga to a PDF.
        The pages will be downloaded to the same folder as the PDF.

        Args:
            volume: Which volume to export as a PDF.
            to: Where to output the PDF.
        """

        if any(m is None for m in (fpdf, imagesize, natsort)):
            raise RuntimeError(
                "PDF extension is required for export, install with 'pip install tankobon[pdf]'"
            )

        path = pathlib.Path(to)
        chapters = [
            c for c, v in self.data["chapters"].items() if v["volume"] == volume
        ]

        self.download(path.parent, chapters=chapters)

        _log.info("[pdf] creating pdf for volume %s", volume)
        pdf = fpdf.FPDF()

        for chapter in natsort.natsorted(chapters):
            _log.info("[pdf] adding chapter %s", chapter)

            chapter_path = path / chapter

            for page in natsort.natsorted((str(p) for p in chapter_path.glob("*.*"))):
                _log.debug("[pdf] adding page %s", page)
                pdf.add_page()

                width, height = imagesize.get(page)
                page_width = 210
                page_height = 297
                ratio = min(page_width / width, page_height / height)

                # use ratio to scale correctly
                try:
                    pdf.image(page, 0, 0, w=width * ratio, h=height * ratio)
                except RuntimeError as e:
                    raise RuntimeError(page, e)

        _log.info("[pdf] saving %s.pdf", volume)
        pdf.output(str(path / f"{volume}.pdf"), "F")



# coding: utf8

import html
import json
import re
from urllib.parse import urlparse

from tankobon import manga

BASE_URL = "mangadex.org"
API_URL = f"https://{BASE_URL}/api/v2"
RE_ID = re.compile(r"^/title/(\d+)")


def _parse_id(url: str) -> str:
    return RE_ID.findall(urlparse(url).path)[0]


class Parser(manga.Parser):

    domain = BASE_URL

    def __init__(self, *args, **kwargs):

        super().__init__(*args, **kwargs)

        api_url = self.data.get("api_url")
        if api_url is None:
            api_url = f"{API_URL}/manga/{_parse_id(self.data['url'])}"
            self.data["api_url"] = api_url

        with self.session.get(api_url) as resp:
            self.api_data = resp.json()["data"]

    def chapters(self):

        with self.session.get(f"{self.data['api_url']}/chapters") as response:
            api_chapters = [
                c
                for c in response.json()["data"]["chapters"]
                if c["language"] == "gb"  # FIXME: multi-language support
            ]
            api_chapters.reverse()

        previous_volume = "0"
        for chapter in api_chapters:
            volume = chapter["volume"]
            if not volume:
                volume = previous_volume
            else:
                previous_volume = volume

            if not chapter["chapter"]:
                # oneshot??
                chapter["chapter"] = "0"

            yield {
                "id": chapter["chapter"],
                "title": chapter["title"],
                # NOTE: data_saver is set to true for now (higher-quality image download keeps getting dropped)
                "url": f"{API_URL}/chapter/{chapter['id']}?saver=true",
                "volume": volume,
            }

    def pages(self, soup):
        chapter_data = json.loads(soup.text)["data"]

        chapter_hash = chapter_data["hash"]
        pages = chapter_data["pages"]
        base_url = chapter_data["server"]

        return [f"{base_url}{chapter_hash}/{page}" for page in pages]

    def title(self):
        return self.api_data["title"]

    def description(self):
        return html.unescape(self.api_data["description"])



# coding: utf8

import collections
import functools
import logging

import click
import coloredlogs  # type: ignore

from tankobon import __version__, manga, parsers

click.option = functools.partial(click.option, show_default=True)  # type: ignore

_log = logging.getLogger("tankobon")
VERBOSITY = [
    getattr(logging, level)
    for level in ("CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG")
]

MAX_COLUMNS = 10


@click.group()
@click.version_option(__version__)
@click.option(
    "-v", "--verbose", "verbosity", help="be more chatty", default=4, count=True
)
def cli(verbosity):
    """Manga browser/downloader."""
    # set up logger
    coloredlogs.install(
        level=VERBOSITY[verbosity - 1],
        fmt=" %(levelname)-8s :: %(message)s",
        logger=_log,
    )


def _parse(url, **kwargs):
    with parsers.Cache() as cache, cache.load(url) as parser:
        parser.parse(**kwargs)
        cache.dump(parser)
        return parser.data


def _info(metadata):
    for key in ("title", "url", "description"):
        click.echo(f"{key}: {metadata[key]}\n")

    volumes = collections.defaultdict(list)
    for chapter, data in metadata["chapters"].items():
        volumes[data["volume"]].append(chapter)

    click.echo()

    click.echo("| volumes | chapters")
    for volume, chapters in volumes.items():

        chapters_str = f",\n| {' ' * 7} | ".join(
            ", ".join(chapters[c : c + MAX_COLUMNS])
            for c in range(0, len(chapters), MAX_COLUMNS)
        )
        click.echo("| {:<7} | {}".format(volume, chapters_str))

    click.echo(
        f"summary: {len(volumes)} volume(s), {len(metadata['chapters'])} chapter(s)"
    )


@cli.command()
@click.option("-u", "--url", default="", help="show info on a specific manga")
def info(url):
    """Show info on parsed/downloaded manga."""
    with parsers.Cache() as cache:
        if url:
            data = cache.load_metadata(url)
            if data.get("title") is None:
                click.echo(f"manga not found, run 'tankobon parse {url}' first")
                return

            _info(data)
        else:
            for url, name in cache._index.items():
                click.echo(f"{url}: {name}")


@cli.command()
@click.argument("url")
@click.option(
    "-f", "--force", default=False, is_flag=True, help="ignore existing metadata"
)
def parse(url, force):
    """Parse a manga's metadata from url and cache it on disk."""
    _parse(url, force=force)


@cli.command()
@click.argument("url")
@click.option("-p", "--path", default=".", help="where to download to")
@click.option(
    "-c",
    "--chapters",
    default="all",
    help="chapters to download, seperated by slashes (ignored if export-pdf is specified)",
)
@click.option(
    "-f", "--force", default=False, is_flag=True, help="re-download existing pages"
)
@click.option(
    "-e",
    "--export-pdf",
    default="",
    help="volumes to create pdfs for, seperated by slashes",
)
def download(url, path, chapters, force, export_pdf):
    """Download a manga's pages by name/url."""
    data = _parse(url, force=False, volume=export_pdf or None)

    if chapters == "all":
        chapters_list = list(data["chapters"])
    else:
        chapters_list = chapters.split("/")

    downloader = manga.Downloader(data)

    if export_pdf != "none":
        # download only the chapters in the volume(s)
        for volume in export_pdf.split("/"):
            downloader.export_pdf(volume, path)
    else:
        downloader.download(path, chapters=chapters_list, force=force)


if __name__ == "__main__":
    cli(prog_name="tankobon")



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['tankobon', 'tankobon._parsers']

package_data = \
{'': ['*'],
 'tankobon': ['.ropeproject/*'],
 'tankobon._parsers': ['.ropeproject/*']}

install_requires = \
['beautifulsoup4>=4.9.1',
 'click>=7.1.2',
 'coloredlogs>=14.0',
 'fake-useragent>=0.1.11',
 'filetype>=1.0.7',
 'requests>=2.24.0']

extras_require = \
{'gui': ['PySide6>=6.0.1'],
 'pdf': ['fpdf>=1.7.2', 'imagesize>=1.2.0', 'natsort>=7.1.0']}

entry_points = \
{'console_scripts': ['tankobon = tankobon.cli:cli']}

setup(name='tankobon',
      version='5.0.0b1',
      description='Yet another manga scraper and downloader',
      author='Ong Yong Xin',
      author_email='ongyongxin2020+github@gmail.com',
      url='https://github.com/ongyx/tankobon',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
